@@ -60,6 +60,74 @@ def processivitySubclassification (event, out2Y, out2K, out2S, out2N):
 			for line in event:
 				print("\t".join([x for x in line]), file=out2N)
 
+
+def pullCCF (project, project_path, correction=True):
+	'''
+	Collects the CCFs from the original mutation files. Assumes that these are provided in the last column of each mutation line
+
+	Parameters:
+			 project	->	user provided project name (string)
+		project_path	->	the directory for the given project (string)
+		  correction	->	optional parameter to perform a genome-wide mutational density correction (boolean; default=False)
+
+	Returns:
+		None
+
+	Outputs:
+		New clustered mutation files that contain the CCF for each mutation.
+	'''
+	path_suffix = ''
+	if correction:
+		path_suffix += "_corrected"
+	vcf_path = project_path + "input/"
+	clusteredMutsPath = project_path + "output/vcf_files" + path_suffix + "/" + project + "_clustered/"
+	clusteredMutsFile = project_path + "output/vcf_files" + path_suffix + "/" + project + "_clustered" + "/SNV/" + project + "_clustered.txt"
+	vcf_files = [x for x in os.listdir(vcf_path) if x != ".DS_Store"]
+
+
+	ccfs = {}
+	for vcfFile in vcf_files:
+		sample = vcfFile.split(".")[0]
+		ccfs[sample] = {}
+		with open(vcf_path + vcfFile) as f:
+			for lines in f:
+				if lines[0] == "#":
+					continue
+				lines = lines.strip().split()
+				chrom = lines[0]
+				if len(chrom) > 3:
+					chrom = chrom[3:]
+				pos = lines[1]
+				ref = lines[3]
+				alt = lines[4]
+				try:
+					ccf = float(lines[-1])
+				except:
+					ccf = -1.5
+					# print("There does not seem to be CCF scores in this input file.\n\t", vcfFile)
+					# break
+				keyLine = ":".join([chrom, pos, ref, alt])
+				ccfs[sample][keyLine] = ccf
+
+	with open(clusteredMutsFile) as f, open(clusteredMutsPath + project + "_clustered_vaf.txt", "w") as out:
+		next(f)
+		print("\t".join([x for x in ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "IMD", "VAF/CCF"]]), file=out)
+		for lines in f:
+			lines = lines.strip().split()
+			sample = lines[1]
+			newKey = ":".join([lines[5], lines[6], lines[8], lines[9]])
+
+			try:
+				ccf = ccfs[sample][newKey]
+			except:
+				newKey = ":".join(["chr" + lines[5], lines[6], lines[8], lines[9]])
+				ccf = ccfs[sample][newKey]
+			lines.append(str(ccf))
+			print("\t".join([x for x in lines]), file=out)
+
+
+
+
 def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, correction=True):
 	'''
 	Collects the VAFs from the original mutation files. Assumes that these are provided in the same 
@@ -70,7 +138,7 @@ def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, c
 		project_path	->	the directory for the given project (string)
 			  sanger	->	optional parameter that informs the tool of what format the VAF scores are provided. This is required when subClassify=True (boolean; default=True)
 				TCGA	->	optional parameter that informs the tool of what format the VAF scores are provided. This is required when subClassify=True and sanger=False (boolean; default=False)
-		  standardVC	->	optional parameter that informs the tool of what format the VAF scores are provided. This is required when subClassify=True and sanger=False and TCGA=False and when the data contains VAF formatted in the 7th column as AF=XX (boolean; default=False)
+		  standardVC	->	optional parameter that informs the tool of what format the VAF scores are provided. This is required when subClassify=True and sanger=False and TCGA=False and when the data contains VAF formatted in the 10th column as AF=XX (boolean; default=False)
 		  correction	->	optional parameter to perform a genome-wide mutational density correction (boolean; default=False)
 
 	Returns:
@@ -109,19 +177,14 @@ def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, c
 					except:
 						print("There does not seem to be VAF scores in this Sanger-produced file.\n\t", vcfFile)
 						break
-					keyLine = ":".join([chrom, pos, ref, alt])
-					vafs[sample][keyLine] = vaf
+					if len(ref) == len(alt) and len(ref) > 1:
+						for i in range(len(ref)):
+							keyLine = ":".join([chrom, str(int(pos)+i), ref[i], alt[i]])
+							vafs[sample][keyLine] = vaf
+					else:
+						keyLine = ":".join([chrom, pos, ref, alt])
+						vafs[sample][keyLine] = vaf
 
-		# with open(clusteredMutsFile) as f, open(clusteredMutsPath + project + "_clustered_vaf.txt", "w") as out:
-		# 	next(f)
-		# 	print("HEADER", file=out)
-		# 	for lines in f:
-		# 		lines = lines.strip().split()
-		# 		sample = lines[1]
-		# 		newKey = ":".join([lines[5], lines[6], lines[8], lines[9]])
-		# 		vaf = vafs[sample][newKey]
-		# 		lines.append(str(vaf))
-		# 		print("\t".join([x for x in lines]), file=out)
 
 	elif TCGA:
 		vafs = {}
@@ -141,8 +204,13 @@ def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, c
 						vaf = float(lines[7].split("VAF=")[1].split(";")[0])
 					except:
 						vaf = -1.5
-					keyLine = ":".join([chrom, pos, ref, alt])
-					vafs[sample][keyLine] = vaf
+					if len(ref) == len(alt) and len(ref) > 1:
+						for i in range(len(ref)):
+							keyLine = ":".join([chrom, str(int(pos)+i), ref[i], alt[i]])
+							vafs[sample][keyLine] = vaf
+					else:
+						keyLine = ":".join([chrom, pos, ref, alt])
+						vafs[sample][keyLine] = vaf
 
 	elif standardVC:
 		vafs = {}
@@ -162,13 +230,19 @@ def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, c
 						vaf = float(lines[9].split("AF=")[1].split(";")[0])
 					except:
 						vaf = -1.5
-					keyLine = ":".join([chrom, pos, ref, alt])
-					vafs[sample][keyLine] = vaf
+					if len(ref) == len(alt) and len(ref) > 1:
+						for i in range(len(ref)):
+							keyLine = ":".join([chrom, str(int(pos)+i), ref[i], alt[i]])
+							vafs[sample][keyLine] = vaf
+					else:
+						keyLine = ":".join([chrom, pos, ref, alt])
+						vafs[sample][keyLine] = vaf
 
 
 	with open(clusteredMutsFile) as f, open(clusteredMutsPath + project + "_clustered_vaf.txt", "w") as out:
 		next(f)
-		print("HEADER", file=out)
+		# print("HEADER", file=out)
+		print("\t".join([x for x in ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "IMD", "VAF/CCF"]]), file=out)
 		for lines in f:
 			lines = lines.strip().split()
 			sample = lines[1]
@@ -183,7 +257,7 @@ def pullVaf (project, project_path, sanger=True, TCGA=False, standardVC=False, c
 			print("\t".join([x for x in lines]), file=out)
 
 
-def findClustersOfClusters (project, chrom_based, project_parent_path, windowSize, chromLengths, regions, log_out, genome, processors, imds, correction=True):
+def findClustersOfClusters (project, chrom_based, project_parent_path, windowSize, chromLengths, regions, log_out, genome, processors, imds, correction=True, includedCCFs=False):
 	'''
 	Subclassifies the clustered mutations into different categories including DBS, MBS, omikli, kataegis, and other. This is only performed
 	for simple single base substutions. Indels are not subclassified using this scheme. 
@@ -270,6 +344,8 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 	# Hard-coded cutoffs, which can later be used as additional parameters to the tool.
 	cutoff = 10001 # Min distance required between adjacenet events
 	vaf_cut = 0.1 # Max difference in VAFs of adjacent mutations
+	if includedCCFs:	
+		vaf_cut = 0.25 # Max difference in VAFs of adjacent mutations
 
 	# Load the IMD data structures
 	if chrom_based:
@@ -292,6 +368,7 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 		distances = []
 		count = 1
 		out = open(out_file, 'w')
+		print("\t".join([x for x in ["clust_group", "project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "IMD", "VAF/CCF"]]), file=out)
 		with open(file) as f:
 			next(f)
 			lines = [line.strip().split() for line in f]
@@ -386,19 +463,22 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 		distances_mnv = {}
 		lines = []
 		count = 1
-		with open(out_file) as f, open(out_file2, 'w') as out2, open(out_file3, 'w') as out3, open(out_file4, 'w') as out4, open(out_file5, 'w') as out5, open(out_file6, 'w') as out6, open(out_file7, 'w') as out7, open(out_file8, 'w') as out8, open(out_file9, 'w') as out2Y, open(out_file10, 'w') as out2K, open(out_file11, 'w') as out2S, open(out_file12, 'w') as out2N:
-			print("HEADER", file=out2)
-			print("HEADER", file=out3)
-			print("HEADER", file=out4)
-			print("HEADER", file=out5)
-			print("HEADER", file=out6)
-			print("HEADER", file=out7)
-			print("HEADER", file=out8)
-			print("HEADER", file=out2Y)
-			print("HEADER", file=out2K)
-			print("HEADER", file=out2S)
-			print("HEADER", file=out2N)
+		subclassesHeader = "\t".join([x for x in ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "group", "IMD", "VAF/CCF", "subclass"]])
 
+		with open(out_file) as f, open(out_file2, 'w') as out2, open(out_file3, 'w') as out3, open(out_file4, 'w') as out4, open(out_file5, 'w') as out5, open(out_file6, 'w') as out6, open(out_file7, 'w') as out7, open(out_file8, 'w') as out8, open(out_file9, 'w') as out2Y, open(out_file10, 'w') as out2K, open(out_file11, 'w') as out2S, open(out_file12, 'w') as out2N:
+			print("\t".join([x for x in ["clust_group", "project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "IMD", "VAF/CCF"]]), file=out2)
+			print(subclassesHeader, file=out3)
+			print(subclassesHeader, file=out4)
+			print(subclassesHeader + "\treason", file=out5)
+			print(subclassesHeader, file=out6)
+			print(subclassesHeader, file=out7)
+			print(subclassesHeader, file=out8)
+			print(subclassesHeader, file=out2Y)
+			print(subclassesHeader, file=out2K)
+			print(subclassesHeader, file=out2S)
+			print(subclassesHeader, file=out2N)
+
+			next(f)
 			for line in f:
 				# line = line.strip().split()[1:] 
 				line = line.strip().split()
@@ -948,18 +1028,19 @@ def findClustersOfClusters_noVAF (project, chrom_based, project_parent_path, win
 		distances_mnv = {}
 		lines = []
 		count = 1
+		subclassesHeader = "\t".join([x for x in ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "group", "IMD", "subclass"]])
 		with open(out_file) as f, open(out_file2, 'w') as out2, open(out_file3, 'w') as out3, open(out_file4, 'w') as out4, open(out_file5, 'w') as out5, open(out_file6, 'w') as out6, open(out_file7, 'w') as out7, open(out_file8, 'w') as out8, open(out_file9, 'w') as out2Y, open(out_file10, 'w') as out2K, open(out_file11, 'w') as out2S, open(out_file12, 'w') as out2N:
-			print("HEADER", file=out2)
-			print("HEADER", file=out3)
-			print("HEADER", file=out4)
-			print("HEADER", file=out5)
-			print("HEADER", file=out6)
-			print("HEADER", file=out7)
-			print("HEADER", file=out8)
-			print("HEADER", file=out2Y)
-			print("HEADER", file=out2K)
-			print("HEADER", file=out2S)
-			print("HEADER", file=out2N)
+			print("\t".join([x for x in ["clust_group", "project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "IMD"]]), file=out2)
+			print(subclassesHeader, file=out3)
+			print(subclassesHeader, file=out4)
+			print(subclassesHeader + "\treason", file=out5)
+			print(subclassesHeader, file=out6)
+			print(subclassesHeader, file=out7)
+			print(subclassesHeader, file=out8)
+			print(subclassesHeader, file=out2Y)
+			print(subclassesHeader, file=out2K)
+			print(subclassesHeader, file=out2S)
+			print(subclassesHeader, file=out2N)
 
 			for line in f:
 				# line = line.strip().split()[1:] 
